
import java.io.File;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.zip.ZipFile;
import java.util.Properties;

public class FindClass
{

    public FindClass()
    {
    }

    private static void jarInput(String s, String s1)
    {
        try
        {
            ZipFile zipfile = new ZipFile(s);
            for(Enumeration enumeration = zipfile.entries(); enumeration.hasMoreElements();)
            {
                String s2 = enumeration.nextElement().toString();
                if(!s2.endsWith("/") && !s2.equals("META-INF/MANIFEST.MF"))
                {
                    
	                System.out.println("Searching for " + s2);   
	                System.out.println("*******************************");                 
                    boolean flag = searchForFile(s2, s1);
                    if(!flag)
                        System.out.println(s2 + " not found!");
                }
            }

        }
        catch(Exception exception)
        {
            exception.printStackTrace();
        }
    }

    public static void main(String args[])
    {
        if(args.length == 0)
        {
            printUsage();
            return;
        }
        if(args.length % 2 != 0)
        {
            System.err.println("Illegal command line: Odd number of arguments specified.");
            printUsage();
            return;
        }
        Hashtable hashtable = parseArgs(args);
        if(!validateParameters(hashtable))
        {
            printUsage();
            return;
        }
        if(singleFile)
        {
            System.out.println("Searching for " + searchValue);
            System.out.println("*******************************");
            boolean flag = searchForFile(searchValue, rootSearchDir);
            if(!flag)
                System.out.println(searchValue + " not found!");
        } else
        {
            jarInput(searchValue, rootSearchDir);
        }
    }

    private static Hashtable parseArgs(String as[])
    {
        Hashtable hashtable = new Hashtable();
        for(int i = 0; i < as.length; i += 2)
        {
            String s = as[i];
            String s1 = as[i + 1];
            hashtable.put(s, s1);
        }

        return hashtable;
    }

    private static void printUsage()
    {
        System.out.println("Command usage: java FindClass (-singleFile searchString)|(-inputJar fileName) -searchDir directoryToSearch [-recurse (yes|no)] [-verbose (yes|no)]");
        System.out.println("\nThis program takes either an single search string (-singleFile) or a Jar file and a search directory.");
        System.out.println("If the program is given a single search string it will search all the Jar and zip files in the search");
        System.out.println("directory for entries matching the string *searchString* where the *'s are wildcards.");
        System.out.println("The matching in the search is case sensitive.");
        System.out.println("To avoid getting hits on package names specify the search string with a preceeding '/' and a following '.class' ");
        System.out.println("\nIf the program is given an inputJar it will search every zip and jar file in the search directory for every entry");
        System.out.println("in the input jar with the exception of the manifest file and directories.");
        System.out.println("Optional parameters:");
        System.out.println("\t-recurse (yes|no) - allows you to specify whether or not to recurse to subdirectories of the search directory.");
        System.out.println("\t\tThe default is yes");
        System.out.println("\t-verbose (yes|no) - allows you to specify whether to output information on how the search is progressing.");
        System.out.println("\t\tThe default is no.");
        System.out.println("\nUsage Examples:");
        System.out.println("java FindClass -singleFile com/ibm/ejs/sm/server/AdminServer. -searchDir c:\\websphere\\appserver\\lib");
        System.out.println("\tThis will search for the AdminServer class in the specified package in all the jars and zips in the search directory.");
        System.out.println("java FindClass -singleFile /Servlet.class -searchDir c:\\websphere\\appserver\\lib");
        System.out.println("\tThis will search for all classes named Servlet in all the zips and jars in the search directory.");
        System.out.println("java FindClass -singleFile Servlet -searchDir c:\\websphere\\appserver\\lib");
        System.out.println("\tThis will search for all files whose name and/or package includes \"Servlet\" in all the zips and jars in the search directory.");
        System.out.println("java FindClass -inputJar PQ48816.jar -searchDir c:\\websphere\\appserver\\lib");
        System.out.println("\tThis will search for all files in PQ48816.jar in all the jars and zips in the search directory.");
        System.out.println("\nIf you encounter this error while running on an iSeries:");
        System.out.println("\tjava.lang.NoSuchMethodError\n\tjava/lang/Error.<init>()V+1 (Error.java:43)\n\tFindClass.searchForFile(Ljava/lang/String;Ljava/lang/String;)Z+0 (FindClass.java:71)\n\tFindClass.main([Ljava/lang/String;)V+0 (FindClass.java:15)");
        System.out.println("Add -Djava.version=1.2 to the command line like this: java -Djava.version=1.2 FindClass ...");
    }

    private static boolean searchForFile(String s, String s1)
    {
        if(recurse && verbose)
            System.out.println("\tAnalyzing directory:" + s1);
        try
        {
            boolean flag = false;
            File file = new File(s1);
            File afile[] = file.listFiles();
            if(afile == null)
            {
                System.err.println("Couldn't read from directory: " + s1);
                return false;
            }
            for(int i = 0; i < afile.length; i++)
            {
                if(recurse && afile[i].isDirectory())
                    flag = searchForFile(s, afile[i].toString()) || flag;
                String s2 = afile[i].toString();
                if(s2.endsWith(".jar") || s2.endsWith(".zip"))
                {
                    ZipFile zipfile = new ZipFile(s2);
                    for(Enumeration enumeration = zipfile.entries(); enumeration.hasMoreElements();)
                    {
                        String s3 = enumeration.nextElement().toString();
                        if(s3.indexOf(s) != -1)
                        {
                            System.out.println("   Found in " + s2 + " as " + s3);
                            flag = true;
                        }
                    }

                }
            }

            return flag;
        }
        catch(java.io.FileNotFoundException e){
	        
	        if(true==verbose){
		        e.printStackTrace();
	        }
	        
	        
        }
        catch(Exception exception)
        {
            exception.printStackTrace();
        }
        return false;
    }

    private static boolean validateParameters(Hashtable hashtable)
    {
        for(Enumeration enumeration = hashtable.keys(); enumeration.hasMoreElements();)
        {
            String s = (String)enumeration.nextElement();
            if(!s.equals("-inputJar") && !s.equals("-singleFile") && !s.equals("-searchDir") && !s.equals("-recurse") && !s.equals("-verbose"))
            {
                System.err.println("Invalid argument: " + s);
                return false;
            }
        }

        String s1         = (String)hashtable.get("-singleFile");
        String s2         = (String)hashtable.get("-inputJar");
        String dotClass   = ".class";
        String slashClass = "/class";
 
        if(s1 != null && s2 != null)
        {
            System.err.println("Illegal command line. You can't specify both singleFile and inputJar");
            return false;
        }
        if(s1 == null && s2 == null)
        {
            System.err.println("Illegal command line. You must specify either singleFile or inputJar");
            return false;
        }
        if(s1 == null)
        {
            singleFile = false;            
            searchValue = s2;
        } else
        {
	      //System.out.println("singleFile="+s1);
            searchValue = s1.replace('.', '/'); 
            int endsInSlashClass = searchValue.length() - 6;
            if ( searchValue.lastIndexOf(slashClass) == endsInSlashClass ){
               searchValue = searchValue.substring(0,endsInSlashClass) + dotClass;
            }   
            //System.out.println("\nsearchValue = "+searchValue+"\n");
        }
        s1 = (String)hashtable.get("-recurse");
        if(s1 != null && s1.equalsIgnoreCase("no"))
            recurse = false;
        s1 = (String)hashtable.get("-verbose");
        if(s1 != null && s1.equalsIgnoreCase("yes"))
            verbose = true;
        String s3 = (String)hashtable.get("-searchDir");
        if(s3 == null)
        {
	        File dir = new File (".");                        	        
	        System.out.println("Search directory is not specified. The current directory ('.') will be searched.");
            try {
				System.out.println("Current Directory is "+dir.getCanonicalPath());
				rootSearchDir = dir.getCanonicalPath();
			} catch (java.io.IOException e) {				
				e.printStackTrace();
				return false;
			}
			                       
            return true;
        } else
        {
            rootSearchDir = s3;
            return true;
        }
    }

    private static boolean singleFile = true;
    private static boolean recurse = true;
    private static String rootSearchDir = null;
    private static String searchValue = null;
    private static boolean verbose = false;

}
